#ifndef __SOFTSPI_H
#define __SOFTSPI_H

#include "main.h"
#include "dwt_stm32_delay.h"
#include "gpio.h"

// SPI 频率选择
#define SPI_FREQ_2_5KHZ 100 // SPI 频率 2.5KHz
#define SPI_FREQ_5KHZ 50    // SPI 频率 5KHz
#define SPI_FREQ_10KHZ 25   // SPI 频率 10KHz
#define SPI_FREQ_20KHZ 12   // SPI 频率 20KHz  (实际约19.4KHz)
#define SPI_FREQ_50KHZ 5    // SPI 频率 50KHz  (实际约42KHz)
#define SPI_FREQ_100KHZ 2   // SPI 频率 100KHz (实际约80KHz)
#define SPI_FREQ_200KHZ 1   // SPI 频率 200KHz (实际约120KHz)

typedef struct
{
    // SPI SCLK 引脚
    GPIO_TypeDef *SCLK_GPIO;
    uint32_t SCLK_Pin;

    // SPI MOSI 引脚
    GPIO_TypeDef *MOSI_GPIO;
    uint32_t MOSI_Pin;

    // SPI MISO 引脚
    GPIO_TypeDef *MISO_GPIO;
    uint32_t MISO_Pin;

    // SPI CS 引脚
    GPIO_TypeDef *CS_GPIO;
    uint32_t CS_Pin;

    // SPI 时钟延时（频率）
    uint32_t Delay_Time;

    // SPI 时钟极性
    uint8_t CPOL;

    // SPI 时钟相位
    uint8_t CPHA;

} SoftSPI_TypeDef;

extern SoftSPI_TypeDef SoftSPI1;

#define SCLK_Set HAL_GPIO_WritePin(SoftSPIx->SCLK_GPIO, SoftSPIx->SCLK_Pin, SoftSPIx->CPOL ? GPIO_PIN_RESET : GPIO_PIN_SET)
#define SCLK_Clr HAL_GPIO_WritePin(SoftSPIx->SCLK_GPIO, SoftSPIx->SCLK_Pin, SoftSPIx->CPOL ? GPIO_PIN_SET : GPIO_PIN_RESET)

#define MOSI_Set HAL_GPIO_WritePin(SoftSPIx->MOSI_GPIO, SoftSPIx->MOSI_Pin, GPIO_PIN_SET)
#define MOSI_Clr HAL_GPIO_WritePin(SoftSPIx->MOSI_GPIO, SoftSPIx->MOSI_Pin, GPIO_PIN_RESET)

#define MISO_Read HAL_GPIO_ReadPin(SoftSPIx->MISO_GPIO, SoftSPIx->MISO_Pin)

#define CS_Set HAL_GPIO_WritePin(SoftSPIx->CS_GPIO, SoftSPIx->CS_Pin, GPIO_PIN_SET)
#define CS_Clr HAL_GPIO_WritePin(SoftSPIx->CS_GPIO, SoftSPIx->CS_Pin, GPIO_PIN_RESET)

#define Delay DWT_Delay_us(SoftSPIx->Delay_Time)

void softSPI_Init(SoftSPI_TypeDef *SoftSPIx);

void SoftSPI_CS_High(SoftSPI_TypeDef *SoftSPIx);
void SoftSPI_CS_Low(SoftSPI_TypeDef *SoftSPIx);

void SoftSPI_WriteReadBuff(SoftSPI_TypeDef *SoftSPIx, uint8_t *pWrite, uint8_t *pRead, uint32_t len);

#endif
